/**
* @file		AHCustomStream.h
*
* @brief	Read/WriteJX^}CYłStreamNXB.NET/JavaXg[c++ɓn̂Ɏg
*
* @author	Taich Hiroshika
* @date		2008-1-24 쐬
*
* $Id: AHCustomStream.h 1595 2013-05-30 02:12:47Z tnishimura $
*
* Copyright (C) 2002-2003, Antenna House, Inc.  All Rights Reserved.
*/

#ifndef AHCUSTOMSTREAM_H__
#define AHCUSTOMSTREAM_H__

#include "AHBase.h"
#ifndef CCCTRL_NOAHCUSTOMSTREAM

#ifdef AH_WIN_MSC_
#pragma warning(push)
#pragma warning(disable: 4250)
#pragma warning(disable: 4275)
#endif

#include <iostream>

namespace AHCommonNS {

/**
 * @class AHCustomStreamController
 *
 */
class AHCOMMON_API AHCustomStreamController
{
public:
	AHCustomStreamController() {}
	virtual ~AHCustomStreamController() {}

	virtual bool canRead() = 0;
	virtual bool canWrite() = 0;
	virtual int read(char* pBuf, int len) = 0;
	virtual int write(char* pBuf, int len) = 0;
};

/**
 * @class AHCustomStreamBuf
 *
 */
class AHCOMMON_API AHCustomStreamBuf : public std::streambuf {
private:
	enum {
		bufferSize		= 0x2000,
		backBufferSize	= 10
	};
	char* m_streamBuffer;
	AHCustomStreamController* m_controller;
	int m_ptr;

public:
	AHCustomStreamBuf(AHCustomStreamController *controller);
	virtual ~AHCustomStreamBuf(void);

	bool is_open(void) const;

protected:
	virtual int_type underflow(void);
	virtual int_type overflow(int_type c);
	virtual std::ios::pos_type seekoff(off_type off, std::ios_base::seekdir way, std::ios_base::openmode which);

private:
	int flushBuffer();
};

/**
 * @class AHCustomStream
 *
 */
class AHCOMMON_API AHCustomStream : public std::basic_iostream<char> {
private:
	AHCustomStreamBuf		m_streamBuf;

public:
#ifndef AH_SOLARIS_ //SPARC_CC_
	// std::istream ƐeNXƔFȂH
	AHCustomStream(AHCustomStreamController *controller) : m_streamBuf(controller), std::basic_iostream<char>(&m_streamBuf) {}
#else
	AHCustomStream(AHCustomStreamController *controller) : m_streamBuf(controller) {init(&m_streamBuf); }
#endif
	bool is_open(void) const	{ return m_streamBuf.is_open(); }
};

} // namespace AHCommonNS

#ifdef AH_WIN_MSC_
#pragma warning(pop)
#endif

#endif //CCCTRL_NOAHCUSTOMSTREAM
#endif // AHCUSTOMSTREAM_H__
